"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var visitor_wrapper_1 = __importDefault(require("../utils/visitor-wrapper"));
var types_1 = require("@babel/types");
var basicMathOperators = ["+", "-", "/", "%", "*"];
var isBasicMathExpression = function (e) {
    return types_1.isBinary(e)
        && basicMathOperators.includes(e.operator);
};
var isSameLogicalExpression = function (a, b) {
    return types_1.isLogicalExpression(a)
        && types_1.isLogicalExpression(b)
        && a.operator == b.operator;
};
var binaryExpressionSides = ["left", "right"];
/**
 * `3 > 2 > 1` -> `(3 > 2) > 1`
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Operator_Precedence
 */
exports.addParenthesesForBinaryExpressions = visitor_wrapper_1.default({
    Binary: function (path) {
        var node = path.node;
        binaryExpressionSides.forEach(function (side) {
            if (types_1.isBinary(node[side])
                && !(isBasicMathExpression(node) && isBasicMathExpression(node[side]))
                && !isSameLogicalExpression(node, node[side])) {
                node[side] = types_1.parenthesizedExpression(node[side]);
            }
        });
    },
});
exports.default = exports.addParenthesesForBinaryExpressions;
